@@ -6,11 +6,10 @@
     'name_es_CO': 'Ventas',
     'name_es_ES': 'Venta',
     'name_fr_FR': 'Vente',
-    'version': '1.4.1',
+    'version': '1.6.0',
     'author': 'B2CK',
     'email': 'info@b2ck.com',
     'website': 'http://www.tryton.org/',
-    'category': 'Sale',
     'description': '''Define sale order.
 Add to product sale informations.
 Define the sale price as the list price.
@@ -100,6 +99,7 @@ Avec la possibilité:
     ],
     'xml': [
         'sale.xml',
+        'configuration.xml',
         'party.xml',
     ],
     'translation': [
