@@ -1,10 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level
-#of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 "Sale"
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard
 from trytond.backend import TableHandler
+from trytond.pyson import If, In, Eval, Get, Or, Not, Equal, Bool, And, \
+        PYSONEncoder
 from decimal import Decimal
 import copy
 
@@ -17,13 +19,17 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
 
     company = fields.Many2One('company.company', 'Company', required=True,
             states={
-                'readonly': "state != 'draft' or bool(lines)",
-            }, domain=["('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))"])
+                'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                    Bool(Eval('lines'))),
+            }, domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     reference = fields.Char('Reference', readonly=True, select=1)
-    description = fields.Char('Description', states={
-        'readonly': "state != 'draft'",
-        })
+    description = fields.Char('Description',
+            states={
+                'readonly': Not(Equal(Eval('state'), 'draft')),
+            })
     state = fields.Selection([
         ('draft', 'Draft'),
         ('quotation', 'Quotation'),
@@ -31,53 +37,55 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         ('done', 'Done'),
         ('cancel', 'Canceled'),
     ], 'State', readonly=True, required=True)
-    sale_date = fields.Date('Sale Date', required=True, states={
-        'readonly': "state != 'draft'",
-        })
+    sale_date = fields.Date('Sale Date', required=True,
+            states={
+                'readonly': Not(Equal(Eval('state'), 'draft')),
+            })
     payment_term = fields.Many2One('account.invoice.payment_term',
             'Payment Term', required=True, states={
-                'readonly': "state != 'draft'",
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
     party = fields.Many2One('party.party', 'Party', change_default=True,
             required=True, states={
-                'readonly': "state != 'draft'",
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             }, on_change=['party', 'payment_term'])
-    party_lang = fields.Function('get_function_fields', type='char',
-            string='Party Language', on_change_with=['party'])
+    party_lang = fields.Function(fields.Char('Party Language',
+        on_change_with=['party']), 'get_function_fields')
     invoice_address = fields.Many2One('party.address', 'Invoice Address',
-            domain=["('party', '=', party)"], states={
-                'readonly': "state != 'draft'",
+            domain=[('party', '=', Eval('party'))], states={
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
     shipment_address = fields.Many2One('party.address', 'Shipment Address',
-            domain=["('party', '=', party)"], states={
-                'readonly': "state != 'draft'",
+            domain=[('party', '=', Eval('party'))], states={
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
     warehouse = fields.Many2One('stock.location', 'Warehouse',
             domain=[('type', '=', 'warehouse')], required=True, states={
-                'readonly': "state != 'draft'",
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
     currency = fields.Many2One('currency.currency', 'Currency', required=True,
         states={
-            'readonly': "state != 'draft' or (bool(lines) and bool(currency))",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                And(Bool(Eval('lines')), Bool(Eval('currency')))),
         })
-    currency_digits = fields.Function('get_function_fields', type='integer',
-            string='Currency Digits', on_change_with=['currency'])
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['currency']), 'get_function_fields')
     lines = fields.One2Many('sale.line', 'sale', 'Lines', states={
-        'readonly': "state != 'draft'",
+        'readonly': Not(Equal(Eval('state'), 'draft')),
         }, on_change=['lines', 'currency', 'party'])
     comment = fields.Text('Comment')
-    untaxed_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Untaxed')
-    tax_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Tax')
-    total_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Total')
+    untaxed_amount = fields.Function(fields.Numeric('Untaxed',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
+    tax_amount = fields.Function(fields.Numeric('Tax',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
+    total_amount = fields.Function(fields.Numeric('Total',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
     invoice_method = fields.Selection([
         ('manual', 'Manual'),
         ('order', 'On Order Confirmed'),
         ('shipment', 'On Shipment Sent'),
     ], 'Invoice Method', required=True, states={
-        'readonly': "state != 'draft'",
+        'readonly': Not(Equal(Eval('state'), 'draft')),
         })
     invoice_state = fields.Selection([
         ('none', 'None'),
@@ -91,16 +99,16 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
             'sale', 'invoice', 'Ignored Invoices', readonly=True)
     invoices_recreated = fields.Many2Many('sale.sale-recreated-account.invoice',
             'sale', 'invoice', 'Recreated Invoices', readonly=True)
-    invoice_paid = fields.Function('get_function_fields', type='boolean',
-            string='Invoices Paid')
-    invoice_exception = fields.Function('get_function_fields', type='boolean',
-            string='Invoices Exception')
+    invoice_paid = fields.Function(fields.Boolean('Invoices Paid'),
+            'get_function_fields')
+    invoice_exception = fields.Function(fields.Boolean('Invoices Exception'),
+            'get_function_fields')
     shipment_method = fields.Selection([
         ('manual', 'Manual'),
         ('order', 'On Order Confirmed'),
         ('invoice', 'On Invoice Paid'),
     ], 'Shipment Method', required=True, states={
-        'readonly': "state != 'draft'",
+        'readonly': Not(Equal(Eval('state'), 'draft')),
         })
     shipment_state = fields.Selection([
         ('none', 'None'),
@@ -108,14 +116,14 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         ('sent', 'Sent'),
         ('exception', 'Exception'),
     ], 'Shipment State', readonly=True, required=True)
-    shipments = fields.Function('get_function_fields', type='many2many',
-            relation='stock.shipment.out', string='Shipments')
-    moves = fields.Function('get_function_fields', type='many2many',
-            relation='stock.move', string='Moves')
-    shipment_done = fields.Function('get_function_fields', type='boolean',
-            string='Shipment Done')
-    shipment_exception = fields.Function('get_function_fields', type='boolean',
-            string='Shipments Exception')
+    shipments = fields.Function(fields.One2Many('stock.shipment.out', None,
+        'Shipments'), 'get_function_fields')
+    moves = fields.Function(fields.One2Many('stock.move', None, 'Moves'),
+            'get_function_fields')
+    shipment_done = fields.Function(fields.Boolean('Shipment Done'),
+            'get_function_fields')
+    shipment_exception = fields.Function(fields.Boolean('Shipments Exception'),
+            'get_function_fields')
 
     def __init__(self):
         super(Sale, self).__init__()
@@ -156,6 +164,10 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                 "SET invoice_method = 'shipment' "\
                 "WHERE invoice_method = 'packing'")
 
+        # Add index on create_date
+        table = TableHandler(cursor, self, module_name)
+        table.index_action('create_date', action='add')
+
     def default_payment_term(self, cursor, user, context=None):
         payment_term_obj = self.pool.get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(cursor, user,
@@ -173,7 +185,6 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         return False
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -222,7 +233,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
     def default_shipment_state(self, cursor, user, context=None):
         return 'none'
 
-    def on_change_party(self, cursor, user, ids, vals, context=None):
+    def on_change_party(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         address_obj = self.pool.get('party.address')
         payment_term_obj = self.pool.get('account.invoice.payment_term')
@@ -255,8 +266,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                     res['payment_term'], context=context).rec_name
         return res
 
-    def on_change_with_currency_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         if vals.get('currency'):
             currency = currency_obj.browse(cursor, user, vals['currency'],
@@ -294,7 +304,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                 res['language'] = sale.party.lang.code
         return res
 
-    def on_change_with_party_lang(self, cursor, user, ids, vals,
+    def on_change_with_party_lang(self, cursor, user, vals,
             context=None):
         party_obj = self.pool.get('party.party')
         if vals.get('party'):
@@ -324,7 +334,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         return res
 
 
-    def on_change_lines(self, cursor, user, ids, vals, context=None):
+    def on_change_lines(self, cursor, user, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         tax_obj = self.pool.get('account.tax')
         invoice_obj = self.pool.get('account.invoice')
@@ -377,7 +387,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                     res['total_amount'])
         return res
 
-    def get_function_fields(self, cursor, user, ids, names, args, context=None):
+    def get_function_fields(self, cursor, user, ids, names, context=None):
         '''
         Function to compute function fields for sale ids
 
@@ -385,7 +395,6 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         :param user: the user id
         :param ids: the ids of the sales
         :param names: the list of field name to compute
-        :param args: optional argument
         :param context: the context
         :return: a dictionary with all field names as key and
             a dictionary as value with id as key
@@ -657,7 +666,7 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                 return False
         return True
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return []
         res = {}
@@ -666,16 +675,12 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
                     + ' - ' + sale.party.rec_name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            names = args[i][2].split(' - ', 1)
-            args2.append(('reference', args[i][1], names[0]))
-            if len(names) != 1 and names[1]:
-                args2.append(('party', args[i][1], names[1]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        names = clause[2].split(' - ', 1)
+        res = [('reference', clause[1], names[0])]
+        if len(names) != 1 and names[1]:
+            res.append(('party', clause[1], names[1]))
+        return res
 
     def copy(self, cursor, user, ids, default=None, context=None):
         if default is None:
@@ -708,13 +713,15 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
         :return: True if succeed
         '''
         sequence_obj = self.pool.get('ir.sequence')
+        config_obj = self.pool.get('sale.configuration')
 
         sale = self.browse(cursor, user, sale_id, context=context)
 
         if sale.reference:
             return True
 
-        reference = sequence_obj.get(cursor, user, 'sale.sale',
+        config = config_obj.browse(cursor, user, 1, context=context)
+        reference = sequence_obj.get_id(cursor, user, config.sale_sequence.id,
                 context=context)
         self.write(cursor, user, sale_id, {
             'reference': reference,
@@ -883,9 +890,9 @@ class Sale(ModelWorkflow, ModelSQL, ModelView):
             vals = moves[line_id]
             vals['shipment_out'] = shipment_id
             move_id = move_obj.create(cursor, 0, vals, context=ctx)
-            sale_line_obj.write(cursor, user, line_id, {
+            sale_line_obj.write(cursor, 0, line_id, {
                 'moves': [('add', move_id)],
-                }, context=context)
+                }, context=ctx)
         shipment_obj.workflow_trigger_validate(cursor, 0, shipment_id,
                 'waiting', context=ctx)
         return shipment_id
@@ -947,55 +954,62 @@ class SaleLine(ModelSQL, ModelView):
         ('comment', 'Comment'),
         ], 'Type', select=1, required=True)
     quantity = fields.Float('Quantity',
-            digits="(16, unit_digits)",
+            digits=(16, Eval('unit_digits', 2)),
             states={
-                'invisible': "type != 'line'",
-                'required': "type == 'line'",
-                'readonly': "not globals().get('_parent_sale')",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'required': Equal(Eval('type'), 'line'),
+                'readonly': Not(Bool(Eval('_parent_sale'))),
             }, on_change=['product', 'quantity', 'unit',
                 '_parent_sale.currency', '_parent_sale.party'])
     unit = fields.Many2One('product.uom', 'Unit',
             states={
-                'required': "product",
-                'invisible': "type != 'line'",
-                'readonly': "not globals().get('_parent_sale')",
-            }, domain=["('category', '=', " \
-                    "(product, 'product.default_uom.category'))"],
-            context="{'category': (product, 'product.default_uom.category')}",
+                'required': Bool(Eval('product')),
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'readonly': Not(Bool(Eval('_parent_sale'))),
+            }, domain=[
+                ('category', '=',
+                    (Eval('product'), 'product.default_uom.category')),
+            ],
+            context={
+                'category': (Eval('product'), 'product.default_uom.category'),
+            },
             on_change=['product', 'quantity', 'unit', '_parent_sale.currency',
                 '_parent_sale.party'])
-    unit_digits = fields.Function('get_unit_digits', type='integer',
-            string='Unit Digits', on_change_with=['unit'])
+    unit_digits = fields.Function(fields.Integer('Unit Digits',
+        on_change_with=['unit']), 'get_unit_digits')
     product = fields.Many2One('product.product', 'Product',
             domain=[('salable', '=', True)],
             states={
-                'invisible': "type != 'line'",
-                'readonly': "not globals().get('_parent_sale')",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'readonly': Not(Bool(Eval('_parent_sale'))),
             }, on_change=['product', 'unit', 'quantity', 'description',
                 '_parent_sale.party', '_parent_sale.currency'],
-            context="{'locations': " \
-                        "_parent_sale.warehouse and [_parent_sale.warehouse] " \
-                        "or False, " \
-                    "'stock_date_end': _parent_sale.sale_date, " \
-                    "'salable': True, " \
-                    "'stock_skip_warehouse': True}")
+            context={
+                'locations': If(Bool(Get(Eval('_parent_sale', {}),
+                    'warehouse')),
+                    [Get(Eval('_parent_sale', {}), 'warehouse')],
+                    []),
+                'stock_date_end': Get(Eval('_parent_sale', {}), 'sale_date'),
+                'salable': True,
+                'stock_skip_warehouse': True,
+            })
     unit_price = fields.Numeric('Unit Price', digits=(16, 4),
             states={
-                'invisible': "type != 'line'",
-                'required': "type == 'line'",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'required': Equal(Eval('type'), 'line'),
             })
-    amount = fields.Function('get_amount', type='numeric', string='Amount',
-            digits="(16, _parent_sale.currency_digits)",
-            states={
-                'invisible': "type not in ('line', 'subtotal')",
-                'readonly': "not globals().get('_parent_sale')",
-            }, on_change_with=['type', 'quantity', 'unit_price',
-                '_parent_sale.currency'])
+    amount = fields.Function(fields.Numeric('Amount',
+        digits=(16, Get(Eval('_parent_sale', {}), 'currency_digits', 2)),
+        states={
+            'invisible': Not(In(Eval('type'), ['line', 'subtotal'])),
+            'readonly': Not(Bool(Eval('_parent_sale'))),
+        }, on_change_with=['type', 'quantity', 'unit_price',
+            '_parent_sale.currency']), 'get_amount')
     description = fields.Text('Description', size=None, required=True)
     note = fields.Text('Note')
     taxes = fields.Many2Many('sale.line-account.tax', 'line', 'tax', 'Taxes',
             domain=[('parent', '=', False)], states={
-                'invisible': "type != 'line'",
+                'invisible': Not(Equal(Eval('type'), 'line')),
             })
     invoice_lines = fields.Many2Many('sale.line-account.invoice.line',
             'sale_line', 'invoice_line', 'Invoice Lines', readonly=True)
@@ -1005,10 +1019,9 @@ class SaleLine(ModelSQL, ModelView):
             'sale_line', 'move', 'Ignored Moves', readonly=True)
     moves_recreated = fields.Many2Many('sale.line-recreated-stock.move',
             'sale_line', 'move', 'Recreated Moves', readonly=True)
-    move_done = fields.Function('get_move_done', type='boolean',
-            string='Moves Done')
-    move_exception = fields.Function('get_move_exception', type='boolean',
-            string='Moves Exception')
+    move_done = fields.Function(fields.Boolean('Moves Done'), 'get_move_done')
+    move_exception = fields.Function(fields.Boolean('Moves Exception'),
+            'get_move_exception')
 
     def __init__(self):
         super(SaleLine, self).__init__()
@@ -1040,8 +1053,7 @@ class SaleLine(ModelSQL, ModelView):
     def default_unit_price(self, cursor, user, context=None):
         return Decimal('0.0')
 
-    def on_change_with_unit_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_unit_digits(self, cursor, user, vals, context=None):
         uom_obj = self.pool.get('product.uom')
         if vals.get('unit'):
             uom = uom_obj.browse(cursor, user, vals['unit'],
@@ -1049,7 +1061,7 @@ class SaleLine(ModelSQL, ModelView):
             return uom.digits
         return 2
 
-    def get_unit_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_unit_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             if line.unit:
@@ -1058,7 +1070,7 @@ class SaleLine(ModelSQL, ModelView):
                 res[line.id] = 2
         return res
 
-    def get_move_done(self, cursor, user, ids, name, args, context=None):
+    def get_move_done(self, cursor, user, ids, name, context=None):
         uom_obj = self.pool.get('product.uom')
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
@@ -1085,7 +1097,7 @@ class SaleLine(ModelSQL, ModelView):
             res[line.id] = val
         return res
 
-    def get_move_exception(self, cursor, user, ids, name, args, context=None):
+    def get_move_exception(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             val = False
@@ -1128,7 +1140,7 @@ class SaleLine(ModelSQL, ModelView):
             ctx2['uom'] = product.sale_uom.id
         return ctx2
 
-    def on_change_product(self, cursor, user, ids, vals, context=None):
+    def on_change_product(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         product_obj = self.pool.get('product.product')
         uom_obj = self.pool.get('product.uom')
@@ -1188,11 +1200,11 @@ class SaleLine(ModelSQL, ModelView):
         vals = vals.copy()
         vals['unit_price'] = res['unit_price']
         vals['type'] = 'line'
-        res['amount'] = self.on_change_with_amount(cursor, user, ids,
-                vals, context=context)
+        res['amount'] = self.on_change_with_amount(cursor, user, vals,
+                context=context)
         return res
 
-    def on_change_quantity(self, cursor, user, ids, vals, context=None):
+    def on_change_quantity(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
 
         if context is None:
@@ -1211,10 +1223,10 @@ class SaleLine(ModelSQL, ModelView):
                 context=ctx2)[vals['product']]
         return res
 
-    def on_change_unit(self, cursor, user, ids, vals, context=None):
-        return self.on_change_quantity(cursor, user, ids, vals, context=context)
+    def on_change_unit(self, cursor, user, vals, context=None):
+        return self.on_change_quantity(cursor, user, vals, context=context)
 
-    def on_change_with_amount(self, cursor, user, ids, vals, context=None):
+    def on_change_with_amount(self, cursor, user, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         if vals.get('type') == 'line':
             if isinstance(vals.get('_parent_sale.currency'), (int, long)):
@@ -1229,7 +1241,7 @@ class SaleLine(ModelSQL, ModelView):
             return amount
         return Decimal('0.0')
 
-    def get_amount(self, cursor, user, ids, name, arg, context=None):
+    def get_amount(self, cursor, user, ids, name, context=None):
         currency_obj = self.pool.get('currency.currency')
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
@@ -1434,29 +1446,30 @@ class Template(ModelSQL, ModelView):
     _name = 'product.template'
 
     salable = fields.Boolean('Salable', states={
-        'readonly': "active == False",
+        'readonly': Not(Bool(Eval('active'))),
         })
     sale_uom = fields.Many2One('product.uom', 'Sale UOM', states={
-        'readonly': "active == False",
-        'invisible': "not salable",
-        'required': "salable",
-        }, domain=["('category', '=', (default_uom, 'uom.category'))"],
-        context="{'category': (default_uom, 'uom.category')}",
+        'readonly': Not(Bool(Eval('active'))),
+        'invisible': Not(Bool(Eval('salable'))),
+        'required': Bool(Eval('salable')),
+        }, domain=[
+            ('category', '=', (Eval('default_uom'), 'uom.category')),
+        ],
+        context={'category': (Eval('default_uom'), 'uom.category')},
         on_change_with=['default_uom', 'sale_uom', 'salable'])
 
     def __init__(self):
         super(Template, self).__init__()
-        if 'not bool(account_category) and bool(salable)' not in \
-                self.account_revenue.states.get('required', ''):
-            self.account_revenue = copy.copy(self.account_revenue)
-            self.account_revenue.states = copy.copy(self.account_revenue.states)
-            if not self.account_revenue.states.get('required'):
-                self.account_revenue.states['required'] = \
-                        "not bool(account_category) and bool(salable)"
-            else:
-                self.account_revenue.states['required'] = '(' + \
-                        self.account_revenue.states['required'] + ') ' \
-                        'or (not bool(account_category) and bool(salable))'
+        self.account_revenue = copy.copy(self.account_revenue)
+        self.account_revenue.states = copy.copy(self.account_revenue.states)
+        required = And(Not(Bool(Eval('account_category'))),
+                Bool(Eval('salable')))
+        if not self.account_revenue.states.get('required'):
+            self.account_revenue.states['required'] = required
+        else:
+            self.account_revenue.states['required'] = \
+                    Or(self.account_revenue.states['required'],
+                            required)
         if 'account_category' not in self.account_revenue.depends:
             self.account_revenue = copy.copy(self.account_revenue)
             self.account_revenue.depends = \
@@ -1476,7 +1489,7 @@ class Template(ModelSQL, ModelView):
             return True
         return False
 
-    def on_change_with_sale_uom(self, cursor, user, ids, vals, context=None):
+    def on_change_with_sale_uom(self, cursor, user, vals, context=None):
         uom_obj = self.pool.get('product.uom')
         res = False
 
@@ -1500,11 +1513,6 @@ Template()
 class Product(ModelSQL, ModelView):
     _name = 'product.product'
 
-    def on_change_with_sale_uom(self, cursor, user, ids, vals, context=None):
-        template_obj = self.pool.get('product.template')
-        return template_obj.on_change_with_sale_uom(cursor, user, ids, vals,
-                context=context)
-
     def get_sale_price(self, cursor, user, ids, quantity=0, context=None):
         '''
         Return the sale price for product ids.
@@ -1608,21 +1616,19 @@ ShipmentOut()
 class Move(ModelSQL, ModelView):
     _name = 'stock.move'
 
-    sale_line = fields.Many2One('sale.line', select=1,
+    sale_line = fields.Many2One('sale.line', 'Sale Line', select=1,
             states={
-                'readonly': "state != 'draft'",
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
-    sale = fields.Function('get_sale', type='many2one',
-            relation='sale.sale', string='Sale',
-            fnct_search='search_sale', select=1)
-    sale_exception_state = fields.Function('get_sale_exception_state',
-            type='selection',
-            selection=[('', ''),
-                       ('ignored', 'Ignored'),
-                       ('recreated', 'Recreated')],
-            string='Exception State')
+    sale = fields.Function(fields.Many2One('sale.sale', 'Sale', select=1),
+            'get_sale', searcher='search_sale')
+    sale_exception_state = fields.Function(fields.Selection([
+        ('', ''),
+        ('ignored', 'Ignored'),
+        ('recreated', 'Recreated'),
+        ], 'Exception State'), 'get_sale_exception_state')
 
-    def get_sale(self, cursor, user, ids, name, arg, context=None):
+    def get_sale(self, cursor, user, ids, name, context=None):
         sale_obj = self.pool.get('sale.sale')
 
         res = {}
@@ -1632,17 +1638,10 @@ class Move(ModelSQL, ModelView):
                 res[move.id] = move.sale_line.sale.id
         return res
 
-    def search_sale(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            field = args[i][0]
-            args2.append(('sale_line.' + field, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_sale(self, cursor, user, name, clause, context=None):
+        return [('sale_line.' + name,) + clause[1:]]
 
-    def get_sale_exception_state(self, cursor, user, ids, name, arg,
-                                 context=None):
+    def get_sale_exception_state(self, cursor, user, ids, name, context=None):
         res = {}.fromkeys(ids, '')
         for move in self.browse(cursor, user, ids, context=context):
             if not move.sale_line:
@@ -1704,12 +1703,11 @@ Move()
 class Invoice(ModelSQL, ModelView):
     _name = 'account.invoice'
 
-    sale_exception_state = fields.Function('get_sale_exception_state',
-            type='selection',
-            selection=[('', ''),
-                       ('ignored', 'Ignored'),
-                       ('recreated', 'Recreated')],
-            string='Exception State')
+    sale_exception_state = fields.Function(fields.Selection([
+        ('', ''),
+        ('ignored', 'Ignored'),
+        ('recreated', 'Recreated'),
+        ], 'Exception State'), 'get_sale_exception_state')
 
     def __init__(self):
         super(Invoice, self).__init__()
@@ -1731,8 +1729,7 @@ class Invoice(ModelSQL, ModelView):
         return super(Invoice, self).button_draft(
             cursor, user, ids, context=context)
 
-    def get_sale_exception_state(self, cursor, user, ids, name, arg,
-                                 context=None):
+    def get_sale_exception_state(self, cursor, user, ids, name, context=None):
         sale_obj = self.pool.get('sale.sale')
         sale_ids = sale_obj.search(
             cursor, user, [('invoices', 'in', ids)], context=context)
@@ -1786,19 +1783,13 @@ class OpenCustomer(Wizard):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
         wizard_obj = self.pool.get('ir.action.wizard')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_party_form'),
-            ('module', '=', 'party'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id,
-                context=context)
+        act_window_id = model_data_obj.get_id(cursor, user, 'party',
+                'act_party_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
         cursor.execute("SELECT DISTINCT(party) FROM sale_sale")
         customer_ids = [line[0] for line in cursor.fetchall()]
-        res['domain'] = str([('id', 'in', customer_ids)])
+        res['pyson_domain'] = PYSONEncoder().encode(
+                [('id', 'in', customer_ids)])
 
         model_data_ids = model_data_obj.search(cursor, user, [
             ('fs_id', '=', 'act_open_customer'),
@@ -1823,7 +1814,7 @@ class HandleShipmentExceptionAsk(ModelView):
 
     recreate_moves = fields.Many2Many(
         'stock.move', None, None, 'Recreate Moves',
-        domain=["('id', 'in', domain_moves)"], depends=['domain_moves'])
+        domain=[('id', 'in', Eval('domain_moves'))], depends=['domain_moves'])
     domain_moves = fields.Many2Many(
         'stock.move', None, None, 'Domain Moves')
 
@@ -1927,7 +1918,8 @@ class HandleInvoiceExceptionAsk(ModelView):
 
     recreate_invoices = fields.Many2Many(
         'account.invoice', None, None, 'Recreate Invoices',
-        domain=["('id', 'in', domain_invoices)"], depends=['domain_invoices'],
+        domain=[('id', 'in', Eval('domain_invoices'))],
+        depends=['domain_invoices'],
         help='The selected invoices will be recreated. '\
             'The other ones will be ignored.')
     domain_invoices = fields.Many2Many(
